/*
 * Copyright © 2008 Red Hat, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Soft-
 * ware"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, provided that the above copyright
 * notice(s) and this permission notice appear in all copies of the Soft-
 * ware and that both the above copyright notice(s) and this permission
 * notice appear in supporting documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
 * ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY
 * RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN
 * THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSE-
 * QUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFOR-
 * MANCE OF THIS SOFTWARE.
 *
 * Except as contained in this notice, the name of a copyright holder shall
 * not be used in advertising or otherwise to promote the sale, use or
 * other dealings in this Software without prior written authorization of
 * the copyright holder.
 *
 * Authors:
 *   Kristian Høgsberg (krh@redhat.com)
 */

#include <fcntl.h>
#include <unistd.h>
#include "xorg-server.h"
#include "xf86.h"
#include "xf86Priv.h"
#include "xf86Parser.h"

#define INTEL_VERSION_MAJOR 2
#define INTEL_VERSION_MINOR 2
#define INTEL_VERSION_PATCH 1

enum {
	FLAG_BATCHBUFFER,
	FLAG_DRI2,
};

static OptionInfoRec stubOptions[] = {
	{ FLAG_BATCHBUFFER, "intel-batchbuffer", OPTV_BOOLEAN, { 0 }, FALSE },
	{ FLAG_DRI2, "DRI2", OPTV_BOOLEAN, { 0 }, FALSE },
	{ -1, NULL, OPTV_NONE, { 0 }, FALSE },
};

extern XF86ConfigPtr xf86configptr;

static const char *
getDriverName(void)
{
	Bool value;
	int cmdline_fd;
	int kmode = 0;
	
	cmdline_fd = open("/proc/cmdline", O_RDONLY);
	if (cmdline_fd >= 0) {
		char buf[255];
		int ret;

		ret = read(cmdline_fd, buf, 255);
		if (ret > 0) {
			if (strstr(buf, "i915.modeset=1")) {
				kmode = 1;
			}
		}
		close(cmdline_fd);
	}

	if (kmode)
		return "intel_batchbuffer";

	xf86ProcessOptions(-1, xf86ConfigLayout.options, stubOptions);
	if (xf86configptr->conf_flags &&
	    xf86configptr->conf_flags->flg_option_lst)
		xf86ProcessOptions(-1,
				   xf86configptr->conf_flags->flg_option_lst,
				   stubOptions);

	if (xf86GetOptValBool(stubOptions, FLAG_BATCHBUFFER, &value) && value)
		return "intel_batchbuffer";
	if (xf86GetOptValBool(stubOptions, FLAG_DRI2, &value) && value)
		return "intel_batchbuffer";

	return "intel_master";
}

static pointer
intelStubSetup(pointer module, pointer options, int *errmaj, int *errmin)
{
	const char *name;

	name = getDriverName();

	return LoadSubModule(module, name, NULL, NULL, NULL, NULL,
			     errmaj, errmin);
}

static XF86ModuleVersionInfo intelStubVersRec = {
	"intel",
	MODULEVENDORSTRING,
	MODINFOSTRING1,
	MODINFOSTRING2,
	XORG_VERSION_CURRENT,
	INTEL_VERSION_MAJOR, INTEL_VERSION_MINOR, INTEL_VERSION_PATCH,
	ABI_CLASS_VIDEODRV,
	ABI_VIDEODRV_VERSION,
	MOD_CLASS_VIDEODRV,
	{0, 0, 0, 0}
};

/* We can't call this struct intelStubModuleData, since the loader
 * will look for "module name" + ModuleData. */

_X_EXPORT XF86ModuleData intelModuleData = {
	&intelStubVersRec,
	intelStubSetup,
	NULL
};

/* And of course, can't forget both i810 users */
static XF86ModuleVersionInfo i810StubVersRec = {
	"i810",
	MODULEVENDORSTRING,
	MODINFOSTRING1,
	MODINFOSTRING2,
	XORG_VERSION_CURRENT,
	INTEL_VERSION_MAJOR, INTEL_VERSION_MINOR, INTEL_VERSION_PATCH,
	ABI_CLASS_VIDEODRV,
	ABI_VIDEODRV_VERSION,
	MOD_CLASS_VIDEODRV,
	{0, 0, 0, 0}
};

static pointer
i810StubSetup(pointer module, pointer options, int *errmaj, int *errmin)
{
	return LoadSubModule(module, "intel_master", NULL, NULL, NULL, NULL,
			     errmaj, errmin);
}

_X_EXPORT XF86ModuleData i810ModuleData = {
	&i810StubVersRec,
	i810StubSetup,
	NULL
};
